/*
 * Growing a plant with IOT
 * The idea came from Inbal's mother actually. her mother studied agriculture at the university and always loved plants and flowers , that’s why her house is full of them.
 * As she saw her mother taking care of the plants she always got curious about how to grow them properly and that’s why she wanted to make an easier way to grow them, using the great technology we have today. That’s why we are going to learn how to grow house plants using the CPX, integromat and Blynk.
 * We made 5 scenarios:
 * We learned that the light and temperature that the plant gets is very important and that's why for each such relevant attribute we created 2 scenarios:
 * 1.The lighting level and the temperature level measured from the environment will always be displayed on the Blynk app.
 * 2.Because we cant always watch at the app , we added a scenario that push a notification to our smartphone if the level of the light or the level of the temperature is not suitable for growing the plant.
 * 3.Watering time and fertilization time, we can set a specific watering schedule and a specific fertilization schedule on google calendar and be notified from the CPX whenever the plant need water or fertilizer.
 * 4. Its importent to know also when the soil is too dry or too wet and we can know this by measuring the humidity level of the air. In this scenario we will Display the Moisture Level in the Soil.
 * 5. We can grow houseplants on the balcony as well, but this is not always the ideal place, for example on a summer day when there is a lot of wind, we prefer that the plant be indoors so that it is not destroyed. In this scenario we will measure the wind speed and notifiy when the speed is ideal or not.
*/ 

#include <cmath>
//Blynk related
#define BLYNK_PRINT SerialUSB
#define BLYNK_DEBUG SerialUSB
#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>
char auth[] = "XXX";  //Blynk AUTH TOKEN HERE
char ssid[] = "XXX";  //wifi network name here
char pass[] = "XXX";  //wifi password here
#define EspSerial Serial1
#define ESP8266_BAUD 9600
ESP8266 blynkWifi(&EspSerial);

int moistureLevel;
double humidityLevel;
bool slideSwitch = CircuitPlayground.slideSwitch();
bool moistureModeOn = true;

int windLevel;
int LOWSPEED = 5;
int HIGHSPEED = 15;

int currentLightAmount;
int LOWLIGHT = 420;
int HIGHLIGHT = 630;

float currentTemperatureAmount;
int LOWTEMP = 20;
int HIGHTEMP = 25;

int pixelsNum = 10;
bool wateringNotificationOn = false;
bool fertilizationNotificationOn = false;


void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(9600);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, blynkWifi, ssid, pass);
}

void loop() {
  delay(500);
   
  /*
  * 1 - Show the current amount of light from the area on the app.   
  * * 2 - Check and notify if the current light is suitable or not for growing home plants. 
  */
  currentLightAmount = CircuitPlayground.lightSensor();
  Blynk.virtualWrite(V4, currentLightAmount);
  if(currentLightAmount < LOWLIGHT)    
  {
     notifyAboutLight("light"); 
  }else if(currentLightAmount > HIGHLIGHT)
  {
     notifyAboutLight("dark");
  }

  /*   
  * 1 - Show the current amount of temperature from the area on the app.
  * 2 - Check and notify if the current temperature is suitable or not for growing home plants. 
  */
  currentTemperatureAmount = CircuitPlayground.temperature();
  Blynk.virtualWrite(V5, currentTemperatureAmount); 
  if(currentTemperatureAmount < LOWTEMP)
  {
    notifyAboutTemp("cold"); 
  }else if(currentTemperatureAmount > HIGHTEMP)
  {
    Serial.println("hello");
    notifyAboutTemp("hot");
  }

  //show moisture level on CPX.
  slideSwitch = CircuitPlayground.slideSwitch();
  if(slideSwitch)
  {
    Serial.println("ON");
    showMoistureLevel();
  }else if(!slideSwitch && moistureModeOn)
  {
    CircuitPlayground.clearPixels();
  }

  //Check and notify if the current wind speed is suitable or not for growing the plants outside. 
  Blynk.virtualWrite(V1, windLevel);
  if(windLevel < LOWSPEED && windLevel != 0)
  {
    notifyAboutWind("outside"); 
  }else if(windLevel > HIGHSPEED)
  {
    notifyAboutWind("inside");
  }
}

//V0 - conected to the air humidity from the weather website.
BLYNK_WRITE(V0){
  humidityLevel = param.asDouble();
}

//V1 - conected to the wind speed from the weather website.
BLYNK_WRITE(V1){
  windLevel = param.asInt();
}

//V2 - connected to the watering scheduale with google calender.
BLYNK_WRITE(V2){
  wateringNotificationOn = true;
  CircuitPlayground.clearPixels();
  while(wateringNotificationOn)
  {
    wateringDay();
    if(CircuitPlayground.leftButton() == 1)
    {
      wateringNotificationOn = false;
    }
  }
}

//V3 - connected to the fertilization scheduale with google calender.
BLYNK_WRITE(V3){
  fertilizationNotificationOn = true;
  CircuitPlayground.clearPixels();
  while(fertilizationNotificationOn)
  {
    fertilizationDay();
    if(CircuitPlayground.rightButton() == 1)
    {
      fertilizationNotificationOn = false;
    }
  }
}

void showMoistureLevel(){
  moistureLevel = round(humidityLevel / 10) - 1;
  for(int i = 0; i < moistureLevel; i++){
    CircuitPlayground.setPixelColor(i, 139, 69, 19);
  }
}

//CPX - watering day
void wateringDay(){
  for(int i = 0; i < pixelsNum; i++){
    CircuitPlayground.setPixelColor(i, 0, 0, 139);
    CircuitPlayground.playTone(200, 100);
    delay(300);
  }
  CircuitPlayground.clearPixels();
  delay(1000);
}

//CPX - fertilization day
void fertilizationDay(){
  for(int i = 0; i < pixelsNum; i++){
    CircuitPlayground.setPixelColor(i, 34, 139, 34);
    CircuitPlayground.playTone(200, 100);
    delay(300);
  }
  CircuitPlayground.clearPixels();
  delay(1000);
}

//notify if the light is sutiable or not.
void notifyAboutLight(String lightStatus){
  String warningMessage;
  if(lightStatus == "light"){
    warningMessage = "Warning : The light is too low,\n Please put the plant in a place with more light";  
  }
  else{
    warningMessage = "Warning : The light is too high,\n Please put the plant in a darker place";
  }
  Blynk.virtualWrite(V4, warningMessage);
}

//notify if the temperature is sutiable or not.
void notifyAboutTemp(String tempStatus){
  String warningMessage;
  if(tempStatus == "cold"){
    warningMessage = "Warning : The temperature is too low,\n Please put the plant in a warmer place";  
  }
  else{
    warningMessage = "Warning : The temperature is too high,\n Please put the plant in a cooler place";
  }
  Blynk.virtualWrite(V5, warningMessage);
}

//notify if the wind speed outside is sutiable or not.
void notifyAboutWind(String areaStatus){
  String warningMessage;
  if(areaStatus == "inside"){
    warningMessage = "Warning : The wind speed is too high,\n Please put the plant indoors";  
  } 
  else 
  {
    if(areaStatus == "outside")
    {
      warningMessage = "Notice : The wind speed is low,\n You can put the plant outside";
    }
  }
  Blynk.virtualWrite(V1, warningMessage);
}
